package stonesoup;

import java.util.Set;
import studentmodel.Command;
import studentmodel.Config;
import studentmodel.Course;

public class ProcessExecuteCmds {
    
    public Config processCommandLine(String[] args, Config config) 
    {        
        boolean success = true;
        boolean foundStudent = false;
        boolean foundCourse = false;
        boolean foundPassword = false;

        final String command = args.length > 0 ? args[0] : null;
        
        
    	
    	
        
        if(command == null) {
            success = false;
        } else {
            config.setCommand(command.toLowerCase());
        }

        
        
        for(int i = 1; i < args.length; i++) {
            final String arg = args[i].toLowerCase();
            String val = null;
            if(args.length > i + 1){
                val = args[++i];
            }
           
            
            if(arg.equalsIgnoreCase("-f") && val != null) {
                config.setFirstName(val);
            }
            else if(arg.equalsIgnoreCase("-l") && val != null){
                config.setLastName(val);
            }
            else if(arg.equalsIgnoreCase("-c") && val != null){
            	config.setCourseNumber(val);
            	foundCourse = true;
            }
            else if(arg.equalsIgnoreCase("-s") && val != null){
            	config.setStudentId(val);
            	foundStudent = true;          			
            }
            
            else if(arg.equalsIgnoreCase("-c") && val != null)
            {
            	config.setCourseNumber(val);
            }
            else if (arg.equalsIgnoreCase("-sl"))
            {
              config.setSizeLimit(val);     			
            }
            else if (arg.equalsIgnoreCase("-cn"))
            {
            	config.setCourseName(val);
            }
            else if (arg.equalsIgnoreCase("-p"))
            {
            	config.setPassword(val);
            	foundPassword = true;
            }
            else if (arg.equalsIgnoreCase("-t"))
            {
              config.setTeacherId(val);  			
            }
            else if(val != null)
            {
            	
            	config.setInstructions(val);
            }                    
        }
        
        if(!success 
                 || (command.equals(Command.liststudentcourses) && (!foundStudent ||!foundPassword))
                || (command.equals(Command.register) && (!foundStudent || !foundCourse ||!foundPassword))
                || (command.equals(Command.unregister) && (!foundStudent || !foundCourse || !foundPassword))                
                 
          )
        {
        	System.err.print("wrong command/arguments parsed");
        }
		return config;    
    }
    
    
    public void execute(Config config) throws Exception{
    	
    	
    	Command command = Command.UNKNOWN;
    	try {
			command = Command.valueOf(config.getCommand());
		} catch (IllegalArgumentException e) {
			
			command = Command.UNKNOWN;
		}
		
		
		final RegistrationDao service = new RegistrationDao();
    	switch (command) {
    	case register:
    		
            service.addRegistration(config);
            break;
    	case liststudentcourses:
    		
            final Set<Course> rlist = service.getRegistrationsForStudent(config);
  		    service.printRegistrations(rlist);
            break;
    	case unregister:
        	 
            service.deleteRegistration(config);
            break;
        default:
        	
        	System.err.print("Error Unknown Command");
        	System.out.print("Error Unknown Command");
          usage();
            System.exit(1);
			break;
    	}        
    }    

    
    public static void usage() 
    {
   	  System.err.print("School Manager Help: [command] [options {-s -f -l -p -c -t -cn -sl}]\n" +
         "" + Command.liststudentcourses + " -s [studentid]  -p [password]\n" +
         "" + Command.register + " -s [studentid] -c [courseabrv] -p [password]\n" +
         "" + Command.unregister + " -s [studentid] -c [courseabrv] -p [password]\n" +

         "");
    }
}
