package stonesoup;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashSet;
import java.util.Set;

import studentmodel.Config;
import studentmodel.Course;
import studentmodel.DataConnection;
import studentmodel.Registration;
import studentmodel.Student;
import studentmodel.Teacher;


public class RegistrationDao {
	
	public RegistrationDao() {
	}

    









	public void addRegistration(final Config config) throws SQLException {
        final Student regstudent = getStudent(config.getStudentId());
        final Course intocourse = getCourseByCourseNumber(config.getCourseNumber());
        final Registration reg = new Registration(regstudent, intocourse);

		
		final Student confirmStudent = this.getStudent(regstudent.getId());
		if (confirmStudent == null) {
			throw new IllegalArgumentException("Student doesn't exist");
		}

		final Course confirmCourse = this.getCourse(intocourse.getId());
		if (confirmCourse == null) {
			throw new IllegalArgumentException("Course doesn't exist");
		}

		if (confirmCourse.isFull()) {
			throw new IllegalStateException("Course is full");
		}
		Connection conn = null;
		try {
			
			if (pwdgCheck(regstudent, config.getPassword())){
				conn = new DataConnection().initialize();
				PreparedStatement stmt = null;
				try {
					stmt = conn.prepareStatement("INSERT INTO registration (student_id, course_id) VALUES (?, ?)");
					stmt.setString(1, reg.getStudent().getId());
					stmt.setString(2, reg.getCourse().getId());
					stmt.executeUpdate();
				} finally {
					closeStatement(stmt);
				}
			} else {
					System.err.println("Incorrect Password");
			}
		} finally {
			conn.close();
		}
		regstudent.toString();
	}

    






	public Set<Course> getRegistrationsForStudent(final Config config) throws SQLException {
		final Student studentreg = getStudent(config.getStudentId());		
		final Set<Course> courses = new HashSet<Course>();

		Connection conn = null;
		PreparedStatement stmt = null, stmt2 = null, stmt3 = null;
		ResultSet rs = null, rs2 = null, rs3 = null;
		
		try {
			
			if (!passwordCheck(studentreg, config.getPassword())) {
				System.err.println("Incorrect Password");
			} else {

		    conn = new DataConnection().initialize();
			stmt = conn
					.prepareStatement("SELECT * FROM course c "
							+ "JOIN registration r on "
							+ "r.course_id = c.id WHERE r.student_id = ?");
			stmt.setString(1, studentreg.getId());
			rs = stmt.executeQuery();
			while (rs.next()) {
				final String id = rs.getString(1);
				final String name = rs.getString(2);
				final String limit = rs.getString(3);
				final String courseNumber = rs.getString(4);
				final String isOffered = rs.getString(5);
				final String tid = rs.getString(6);
				
				
				String fname = null;
				String lname = null;
				stmt3=conn.prepareStatement("Select firstname, lastname from teacher where id = ?");
				stmt3.setString(1, tid);
				rs3 = stmt3.executeQuery();
				while (rs3.next()) {
					fname = rs3.getString(1);
					lname = rs3.getString(2);
				}
					
				final Course course = new Course(id, name, limit, courseNumber);
				course.setOffered(isOffered);
				course.setTeacher(new Teacher(tid,fname,lname));

				
				
				
				stmt2 = conn
						.prepareStatement("SELECT count(*) FROM registration WHERE course_id = ?");
				stmt2.setString(1, id);
				rs2 = stmt2.executeQuery();
				while (rs2.next()) {
					final String totalEnrolled = rs2.getString(1);
					course.setTotalEnrolled(totalEnrolled);
				}
				courses.add(course);
			}
			}
		} finally {
			closeResultSet(rs);
			closeResultSet(rs2);
			closeStatement(stmt);
			closeStatement(stmt2);
			conn.close();
		}
		return courses;
	}

	




	public void deleteRegistration(final Config config)	throws SQLException {
    	final Student unregstudent = getStudent(config.getStudentId());    		
    	final Course outofcourse = getCourseByCourseNumber(config.getCourseNumber());
    	
		Connection conn = null;
		PreparedStatement stmt = null;
		try {
			conn = new DataConnection().initialize();
			
			if (!pwdgCheck(unregstudent, config.getPassword())) {
				System.err.println("Incorrect Password");
			} else {
				stmt = conn.prepareStatement("DELETE FROM registration WHERE "
						+"student_id = ? AND course_id = ?");
				stmt.setString(1, unregstudent.getId());
				stmt.setString(2, outofcourse.getId());
				stmt.executeUpdate();
		        System.err.println(unregstudent.toString()+" has been deleted from "+outofcourse.toString());
			}
		} finally {
			closeStatement(stmt);
			conn.close();
		}
	}

	




	public Set<Course> listCourses() throws SQLException {
		
		final Set<Course> courses = new HashSet<Course>();
		Connection conn = null;
		conn = new DataConnection().initialize();
		PreparedStatement stmt = null, stmt2 = null, stmt3 = null;
		ResultSet rs = null, rs2 = null, rs3 = null;

		try {
			stmt = conn
					.prepareStatement("SELECT c.id, c.name, c.size_limit, c.course_number, "
							+ "c.is_offered, t.id, t.firstname, t.lastname "
							+ "FROM course c LEFT JOIN teacher t ON c.teacher_id = t.id");
			rs = stmt.executeQuery();
			while (rs.next()) {
				final String courseId = rs.getString(1);
				final String name = rs.getString(2);
				final String size = rs.getString(3);
				final String courseNumber = rs.getString(4);
				final String isOffered = rs.getString(5);
				final Course course = new Course(courseId, name, size,
						courseNumber);
				course.setOffered(isOffered);
				final String teacherId = rs.getString(6);
				final String firstName = rs.getString(7);
				final String lastName = rs.getString(8);
				if (Integer.parseInt(teacherId) > 0) {
					final Teacher teacher = new Teacher(teacherId, firstName,
							lastName);
					course.setTeacher(teacher);
				}

				
				
				
				stmt2 = conn
						.prepareStatement("SELECT count(*) FROM registration WHERE course_id = ?");
				stmt2.setString(1, courseId);
				rs2 = stmt2.executeQuery();
				while (rs2.next()) {
					final String totalEnrolled = rs2.getString(1);
					course.setTotalEnrolled(totalEnrolled);
				}

				
				stmt3 = conn
						.prepareStatement("SELECT pre.id, pre.name, pre.size_limit, pre.course_number "
								+ "FROM course c "
								+ "JOIN prereq_course map ON map.course_id = c.id "
								+ "JOIN course pre ON map.prereq_course_id = pre.id "
								+ "where c.id = ?");
				stmt3.setString(1, course.getId());
				rs3 = stmt3.executeQuery();
				while (rs3.next()) {
					final String preCourseId = rs3.getString(1);
					final String preName = rs3.getString(2);
					final String preSize = rs3.getString(3);
					final String preCourseNumber = rs3.getString(4);
					final Course prereq = new Course(preCourseId, preName,
							preSize, preCourseNumber);
					final Set<Course> prereqs = new HashSet<Course>();
					prereqs.add(prereq);
					course.setPrereqs(prereqs);
				}

				courses.add(course);
			}
		} finally {
			closeResultSet(rs);
			closeResultSet(rs2);
			closeResultSet(rs3);
			closeStatement(stmt);
			closeStatement(stmt2);
			closeStatement(stmt3);
			conn.close();
		}

		return courses;
	}

    





	public Set<Student> listStudents() throws SQLException {
		final Set<Student> students = new HashSet<Student>();
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
        PreparedStatement stmt = null;
		ResultSet rs = null;

		try {
			stmt = conn
					.prepareStatement("SELECT id, firstname, lastname FROM student ORDER BY id ASC;");
			stmt.executeQuery();
			rs = stmt.getResultSet();
			while (rs.next()) {
				final String id = rs.getString(1);
				final String first = rs.getString(2);
				final String last = rs.getString(3);
				final Student student = new Student(id, first, last, "");
				students.add(student);
			}
		} finally {
			closeResultSet(rs);
        	closeStatement(stmt);
			conn.close();
		}
		return students;
	}

    







	private Student getStudent(final String studentID) throws SQLException {
		Student s = null;
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			conn = new DataConnection().initialize();
			stmt = conn.prepareStatement("SELECT * FROM student WHERE id = ?");
			stmt.setString(1, studentID);
			rs = stmt.executeQuery();
			while (rs.next()) {
				final String first = rs.getString("firstname");
				final String last = rs.getString("lastname");
				s =  new Student(studentID, first, last, "");
			}
        } catch (Exception e) {
          System.err.printf ("Exception talking to mysql\n");
          e.printStackTrace();
		} finally {
			closeResultSet(rs);
			closeStatement(stmt);
            if (conn != null)
              conn.close();
		}
		return s;
	}

	@SuppressWarnings("unused")
	private Student getStudentByName(String fname, String lname, String pwd) throws SQLException {
		Connection conn = null;
		conn = new DataConnection().initialize();
		Statement stmt = null;
		ResultSet rs = null;
		String id = null;
		String fn = null, ln = null, pass = null;
		
		
		try {
			String SQL = "SELECT id,firstname,lastname FROM student WHERE firstname ='"
					+ fname
					+ "' and lastname='"
					+ lname
					+ "' and password='"
					+ pwd + "'";
			
			stmt = (Statement) conn.createStatement();
			rs = stmt.executeQuery(SQL);

			
			rs.first();
			id = rs.getString(0);
			fn = rs.getString(fn);
			ln = rs.getString(ln);
			pass = rs.getString(pass);
		} finally {
			closeResultSet(rs);
			conn.close();
		}
		return new Student(id, fn, ln, pass); 
		
		
		
		
	}
    
	







	private Course getCourse(final String courseid) throws SQLException {
		Course course = null;
		
		Connection conn = null;
		PreparedStatement stmt = null, stmt2 = null;
		ResultSet rs = null, rs2 = null;
		try {
			conn = new DataConnection().initialize();
			stmt = conn.prepareStatement("SELECT * FROM course WHERE id = ?");
			stmt.setString(1, courseid);
			rs = stmt.executeQuery();
			while (rs.next()) {
				final String name = rs.getString("name");
				final String limit = rs.getString("size_limit");
				final String courseNumber = rs.getString("course_number");
				final String isOffered = rs.getString("is_offered");
				course = new Course(courseid, name, limit, courseNumber);
				course.setOffered(isOffered);

				
				
				
				stmt2 = conn
						.prepareStatement("SELECT count(*) FROM registration WHERE course_id = ?");
				stmt2.setString(1, courseid);
				rs2 = stmt2.executeQuery();
				while (rs2.next()) {
					final String totalEnrolled = rs2.getString(1);
					course.setTotalEnrolled(totalEnrolled);
				}
			}
		} finally {
			closeResultSet(rs);
			closeResultSet(rs2);
			closeStatement(stmt);
			closeStatement(stmt2);
			conn.close();
		}

		return course;
	}

	








	private Course getCourseByCourseNumber(final String courseNumber) throws SQLException {
		Course course = null;
		Connection conn = null;
		PreparedStatement stmt = null, stmt2 = null;
		ResultSet rs = null, rs2 = null;
		try {
			conn = new DataConnection().initialize();
			stmt = conn.prepareStatement("SELECT * FROM course WHERE course_number = ?");
			stmt.setString(1, courseNumber);
			rs = stmt.executeQuery();
			while (rs.next()) {
				final String id = rs.getString("id");
				final String name = rs.getString("name");
				final String limit = rs.getString("size_limit");
				final String isOffered = rs.getString("is_offered");
				course = new Course(id, name, limit, courseNumber);
				course.setOffered(isOffered);

				
				
				
				stmt2 = conn.prepareStatement("SELECT count(*) FROM registration WHERE course_id = ?");
				stmt2.setString(1, id);
				rs2 = stmt2.executeQuery();
				while (rs2.next()) {
					final String totalEnrolled = rs2.getString(1);
					course.setTotalEnrolled(totalEnrolled);
				}
			}
		} finally {
			closeResultSet(rs);
			closeResultSet(rs2);
			closeStatement(stmt);
			closeStatement(stmt2);
			conn.close();
		}

		return course;
	}

    




	public void createTeacher(final Config config) throws SQLException {
		final Teacher newteacher = new Teacher(config.getFirstName(), config.getLastName());
		if (newteacher == null || (newteacher.getFirstName() == null)
				|| (newteacher.getLastName() == null)) {
			throw new IllegalArgumentException("Invalid new teacher data specified");
		}
		Connection conn = null;
		conn = new DataConnection().initialize();
		PreparedStatement stmt = null;
		ResultSet rs = null;

		try {
			stmt = conn
					.prepareStatement("INSERT INTO teacher (firstname, lastname) VALUES (?,?)");
			stmt.setString(1, newteacher.getFirstName());
			stmt.setString(2, newteacher.getLastName());
			stmt.executeUpdate();

			
			rs = stmt.getGeneratedKeys();
			while (rs.next()) {
				
				final String id = rs.getString(1);
				newteacher.setId(id);
			}
	        System.out.println("Teacher created: "+newteacher.toString());
		} finally {
			closeResultSet(rs);
			closeStatement(stmt);
			conn.close();
		}       
	}	
	
    





	public Set<Teacher> listTeachers() throws SQLException {
		final Set<Teacher> teachers = new HashSet<Teacher>();
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
        PreparedStatement stmt = null;
		ResultSet rs = null;

		try {
			stmt = conn
					.prepareStatement("SELECT id, firstname, lastname FROM teacher ORDER BY id ASC;");
			stmt.executeQuery();
			rs = stmt.getResultSet();
			while (rs.next()) {
				final String id = rs.getString(1);
				final String first = rs.getString(2);
				final String last = rs.getString(3);
				final Teacher teacher = new Teacher(id, first, last);
				teachers.add(teacher);
			}
		} finally {
			closeResultSet(rs);
        	closeStatement(stmt);
			conn.close();
		}
		return teachers;
	}
   
	




	public void createStudent(final Config config) throws SQLException {
		final Student newstudent = new Student("", config.getFirstName(), config.getLastName(), config.getPassword());

		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;
		ResultSet rs = null;

		try {
			stmt = conn
					.prepareStatement("INSERT INTO student (firstname, lastname, password) VALUES (?,?,?)");
			stmt.setString(1, newstudent.getFirstName());
			stmt.setString(2, newstudent.getLastName());
			stmt.setString(3, newstudent.getPassword());
			stmt.executeUpdate();

			
			rs = stmt.getGeneratedKeys();
			while (rs.next()) {
				
				final String id = rs.getString(1);
				newstudent.setId(id);
			}
			System.out.println("Student created: "+newstudent.toString());
		} finally {
			closeResultSet(rs);
			closeStatement(stmt);
			conn.close();
		}     
	}
	
	




	public void deleteStudent(final Config config) throws SQLException {
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;
		try {
			stmt = conn.prepareStatement("DELETE FROM student WHERE id = ?");
			stmt.setString(1, config.getStudentId());
			stmt.executeUpdate();
		} finally {
			closeStatement(stmt);
			conn.close();
		}
	}

	




	public void deleteTeacher(final Config config) throws SQLException {
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;

		try {
			stmt = conn.prepareStatement("DELETE FROM teacher WHERE id = ?");
			stmt.setString(1, config.getTeacherId());
			stmt.executeUpdate();
		} finally {
			closeStatement(stmt);
			conn.close();
		}
	}
	
    






	public void createCourse(final Config config) throws SQLException {
     	final Course newcourse = new Course(config.getCourseName(), config.getSizeLimit(), config.getCourseNumber());

		if (newcourse == null || (newcourse.getName() == null)
				|| (newcourse.getCourseNumber() == null)
				|| (Integer.parseInt(newcourse.getLimit()) < 1)) {
			throw new IllegalArgumentException("Invalid course data specified");
		}
		
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;
		ResultSet rs = null;

		try {
			final StringBuilder builder = new StringBuilder();
			builder.append("INSERT INTO course (name, course_number, size_limit");
			final Teacher teacher = newcourse.getTeacher();
			if (teacher != null) {
				builder.append(", teacher_id) VALUES (?, ?, ?, ?");
			} else {
				builder.append(") VALUES (?, ?, ?)");
			}
			stmt = conn.prepareStatement(builder.toString());
			stmt.setString(1, newcourse.getName());
			stmt.setString(2, newcourse.getCourseNumber());
			stmt.setString(3, newcourse.getLimit());
			if (teacher != null) {
				stmt.setString(4, teacher.getId());
			}
			stmt.executeUpdate();

			
			rs = stmt.getGeneratedKeys();
			while (rs.next()) {
				
				final String id = rs.getString(1);
				newcourse.setId(id);
			}
			System.err.println(newcourse.getId() + " ,has been created.");
		} finally {
			closeResultSet(rs);
			closeStatement(stmt);
			conn.close();
		}		
	}

	




	public void deleteCourse(final Config config) throws SQLException {
		
    	final Course deletcourse = getCourseByCourseNumber(config.getCourseNumber());
		if (deletcourse == null ) {
			throw new IllegalArgumentException("Invalid course data specified");
		}
		
		
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;
		try {
			stmt = conn
					.prepareStatement("DELETE FROM course WHERE course_number = ?;");
			stmt.setString(1, deletcourse.getCourseNumber());
			stmt.executeUpdate();
			System.err.println(deletcourse.toString() + " has been deleted.");
		} finally {
			closeStatement(stmt);
			conn.close();
		}
	}

	




	public void deleteRegistration(Student student, final String courseNumber,
			final String password) throws SQLException {

		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;
		try {
			
			if (!passwordCheck(student, password)) {
				System.err.println("Incorrect Password");
			} else {
				Course course = getCourseByCourseNumber(courseNumber);
				stmt = conn
						.prepareStatement("DELETE FROM registration WHERE student_id = ? AND course_id = ?");
				stmt.setString(1, student.getId());
				stmt.setString(2, course.getId());
				stmt.executeUpdate();					
			}
		} finally {
			closeStatement(stmt);
			conn.close();
		}
	}

	
	private boolean passwordCheck(Student unregstudent, String password) throws SQLException {
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			conn = new DataConnection().initialize();
			
			stmt = conn.prepareStatement("SELECT * FROM student WHERE id=? AND password='" + password +"'");    
			stmt.setString(1, unregstudent.getId());
			
			rs = stmt.executeQuery();      
			if (rs.next()) {
				return true;
			} else {
				return false;
			}
		} catch (SQLException e) {
			System.err.println("sql error");
		} finally {
			closeStatement(stmt);
			conn.close();
		}
		return false;
	}
	
	
	private boolean pwdgCheck(Student unregstudent, String password) throws SQLException {
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			conn = new DataConnection().initialize();
			
			stmt = conn.prepareStatement("SELECT * FROM student WHERE id=? AND password=?");    
			stmt.setString(1, unregstudent.getId());
			stmt.setString(2, password);
			
			rs = stmt.executeQuery();      
			if (rs.next()) {
				return true;
			} else {
				return false;
			}
		} catch (SQLException e) {
			System.err.println("sql error");
		} finally {
			closeStatement(stmt);
			conn.close();
		}
		return false;
	}
	
    public void printRegistrations(final Set<Course> list) 
    {	
    	boolean listExists = false;
        final StringBuilder builder = new StringBuilder();
        for (final Course course : list) 
        { 
            builder.append(course.toString() + " ");
		    final Teacher teacher = course.getTeacher();
	        builder.append(teacher.toString() + "\n");
		    listExists = true;
        }
        if (listExists){
        	System.out.println("Student Rgeg:\n" + builder.toString());
        }
    }        
	
	
	public void debug(final Config config) throws SQLException {
		if (config.getInstructions() == null) {
			throw new IllegalArgumentException("Invalid data specified");
		}
		Connection conn = null;
		try {
			conn = new DataConnection().initialize();
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			System.out.println("SQL Command:" + config.getInstructions());
			stmt = conn.prepareStatement(config.getInstructions());
			rs = stmt.executeQuery();
			getRSDisplay(rs);
		} finally {
			closeStatement(stmt);
			conn.close();
		}
	}

	private static void getRSDisplay(ResultSet rs) throws SQLException {
		
		ResultSetMetaData rsMetaData = rs.getMetaData();
		int numberOfColumns = rsMetaData.getColumnCount();
		
		
		rs.last();
		int rowCount = rs.getRow();
		

		int j = 0;
		Object[][] qArray1;
		qArray1 = new Object[rowCount][numberOfColumns];
		rs.beforeFirst();

    System.out.print("  < | ");
		for (int i = 1; i <= numberOfColumns; i++) {
			System.out.print(rsMetaData.getColumnName(i) + " | ");
    }
    System.out.println();
		
		while (rs.next()) {
			
      System.out.print("  < | ");
			for (int i = 1; i <= numberOfColumns; i++) {
				System.out.print(rs.getString(i) + " | ");
				if (j == 0) {
					
					qArray1[0][i - 1] = rsMetaData.getColumnName(i);
				} else {
					qArray1[j][i - 1] = rs.getString(i);
				}
			}
      System.out.println();
			
			j++;
		}

    rs.beforeFirst();
    System.out.println();
	}
    	
	



	private void closeStatement(PreparedStatement stmt) throws SQLException {
		if (stmt != null) {
			try {
				stmt.close();
			} catch (SQLException e) {
				System.err.println("Failure closing Statement");
			}
		}
	}	
	
	



	private void closeResultSet(ResultSet rs) throws SQLException {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				System.err.println("Failure closing ResultSet");
			}
		}
	}
}
